import 'dart:convert';
import 'dart:typed_data';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'config_tsg800/barCode.dart';
import 'config_tsg800/configString.dart';

class GertecTsg800 {
  static const MethodChannel _channel = const MethodChannel('gertec_tsg800');

  /// Realiza a leitura da informação contida no cartão.
  ///
  /// Retorna a mensagem contida no cartão.
  static Future<String> get lerCartao async {
    String textoCartao;
    try {
      textoCartao = await _channel.invokeMethod('lerNfc');
    } on PlatformException catch (e) {
      print(e.message);
    }
    return textoCartao;
  }

  /// Finaliza a impressão (ImpressoraOutput)
  static dynamic finalizarImpressao() async {
    return await _channel.invokeMethod('fimimpressao');
  }

  /// Retorna o Status da impressora (True, False)
  static Future<bool> get getStatus async {
    bool status;
    try {
      status = await _channel.invokeMethod('statusImpressora');
    } on PlatformException catch (e) {
      print(e.message);
    }
    return status;
  }

  /// Avanca uma determinada quantidade de linhas no momento da impressão
  static dynamic avancarLinha(int quantidadeLinhas) async {
    try {
      return await _channel.invokeMethod(
          'avancarLinha', <String, dynamic>{"quantLinhas": quantidadeLinhas});
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  /// Realiza a gravação de uma mensagem no cartão.
  ///
  /// Recebe como parâmetro obrigatório uma variável do tipo String, que vai ser o texto que vai ser gravado.
  static Future<dynamic> gravarCartao({@required String mensagemGravar}) async {
    try {
      return await _channel.invokeMethod(
          'gravarNfc', <String, dynamic>{"mensagemGravar": mensagemGravar});
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  /// Realiza a formatação do cartão (Obs: Cuidado para não apagar dados importantes).
  static Future<dynamic> get formatarCartao async {
    try {
      return await _channel.invokeMethod('formatarNfc');
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  /// Realiza um teste de leitura e gravação no Cartão.
  static Future<dynamic> get testeCartao async {
    try {
      return await _channel.invokeMethod('testeNfc');
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  /// Realiza a impressão de texto.
  ///
  /// Recebe como parâmetro obrigatório a mensagem que vai ser impressa.
  ///
  /// Recebe como parâmetro o tamanho da fonte.
  ///
  /// Recebe como parâmetro uma variável da Classe [Alinhamento], que determina onde o texto vai ser impresso.
  ///
  /// Recebe como parâmetro uma variável da Classe [TipofontFamily], que determina a fonte do texto.
  ///
  /// Recebe três parâmetros relacionados a estilo de fonte.
  static Future<dynamic> imprimirTexto({
    @required String texto,
    int tamanhoFonte = 80,
    Alinhamento alinhamento,
    TipofontFamily fontFamily,
    bool negrito = false,
    bool italico = false,
    bool sublinhado = false,
  }) async {
    String ext = "";
    dynamic result = "";
    String fontFamil = fontFamily?.getTipoFontFamily ??
        TipofontFamily.monospace.getTipoFontFamily;
    try {
      if (fontFamil != "MONOSPACE" &&
          fontFamil != "SANS SERIF" &&
          fontFamil != "SERIF") {
        ext = fontFamil
            .substring(
              fontFamil.lastIndexOf('.'),
              fontFamil.length,
            )
            .toString()
            .toLowerCase();
        await convertToBase64(fontFamil).then((value) => fontFamil = value);
      } else {
        ext = "";
      }
      result = await _channel.invokeMethod(
        'imprimirTexto',
        <String, dynamic>{
          "mensagem": texto,
          "alinhar": alinhamento?.getTipoAlinhamento ??
              Alinhamento.centralizado.getTipoAlinhamento,
          "size": tamanhoFonte,
          "font": fontFamil,
          "ext": ext, // verifica qual extensão da fonte passada
          "negrito": negrito,
          "sublinhado": sublinhado,
          "italico": italico
        },
      );
      return result;
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  /// Realiza a impressão de um código de barra.
  ///
  /// Recebe como parâmetro obrigatório a mensagem que vai ser impressa.
  ///
  /// Recebe como parâmetro o height e width, que determinam o tamanho do código de barra.
  ///
  /// Recebe como parâmetro uma variável da Classe [TipoCodigoImpressao], que determina o tipo de código que vai ser impresso.
  static Future<dynamic> imprimirCodigoDeBarra(
      {@required String texto,
      int height = 320,
      int width = 320,
      TipoCodigoImpressao codeType}) async {
    if (texto.isNotEmpty) {
      dynamic result = "";
      try {
        result = await _channel.invokeMethod(
          'imprimirCodigoDeBarra',
          <String, dynamic>{
            "height": height,
            "width": width,
            "barCode": codeType?.getTipoCodigoImpressaoName ??
                TipoCodigoImpressao.qrCode.getTipoCodigoImpressaoName,
            "mensagem": texto
          },
        );
        return result;
      } on PlatformException catch (e) {
        print(e.message);
      }
    }
  }

  /// Realiza a leitura de Codigo de barra.
  ///
  /// Recebe como parâmetro obrigatório uma variável da Classe [TipoCodigoLeitura], que determina qual tipo de código vai ser lido.
  ///
  /// Retorna o valor do codigo barras lido.
  static Future<String> leitorCodigoDeBarra(
      {@required TipoCodigoLeitura tipoLeitura}) async {
    String codigo;
    try {
      codigo = await _channel.invokeMethod(
        "leitorV1",
        <String, dynamic>{
          "tipoLeitura": tipoLeitura?.getTipoCodigoLeituraName ??
              TipoCodigoLeitura.qrCode.getTipoCodigoLeituraName,
        },
      );
    } on PlatformException catch (e) {
      print(e.message);
    }
    return codigo;
  }

  /// Raliza a leitura de Codigo de Barras v2.
  ///
  /// Retorna o valor do codigo barras lido.

  static void leitorCodigoDeBarraV2() async {
    try {
      await _channel.invokeMethod('leitorV2');
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  /// Raliza a impressão de uma imagem.
  /// '''
  /// Recebe como parâmetro o asset Path da imagem
  static Future<dynamic> imprimirImagem(
      {@required String assetPath, int width, int height}) async {
    try {
      dynamic result = "";
      await convertToBase64(assetPath).then(
        (value) async {
          result = await _channel.invokeMethod(
            'imprimirImagem',
            <String, dynamic>{
              "width": width,
              "height": height,
              "codigoImagem": value,
            },
          );
        },
      );
      return result;
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  static Future<String> convertToBase64(String path) async {
    ByteData bytes = await rootBundle.load(path);
    var buffer = bytes.buffer;
    return base64.encode(Uint8List.view(buffer));
  }
}
